#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from gbpclient.gbp.v2_0 import groupbasedpolicy as gbp
from gbpclient.tests.unit import test_cli20


class CLITestV20NetworkServicePolicyJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20NetworkServicePolicyJSON, self).setUp()

    def test_create_nsp_with_mandatory_params(self):
        """network-service-policy-create with mandatory params."""
        resource = 'network_service_policy'
        cmd = gbp.CreateNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                             None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_network_service_policy_with_all_params(self):
        """network-service-policy-create with all params."""
        resource = 'network_service_policy'
        cmd = gbp.CreateNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                             None)
        name = 'myname'
        tenant_id = 'mytenant'
        description = 'Mynsp'
        my_id = 'someid'
        network_svc_params = "type=ip_single,name=vip,value=self_subnet"
        shared = 'true'
        args = ['--tenant_id', tenant_id,
                '--description', description,
                '--network-service-params', network_svc_params,
                '--shared', shared,
                name]
        position_names = ['name', 'description', 'network_service_params']
        net_params = [{"type": "ip_single", "name": "vip",
                       "value": "self_subnet"}]
        position_values = [name, description, net_params]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id, shared=shared)

    def test_list_network_service_policies(self):
        """network-sercvice-policy-list."""
        resources = 'network_service_policies'
        cmd = gbp.ListNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                           None)
        self._test_list_resources(resources, cmd, True)

    def test_list_network_service_policies_with_pagination(self):
        """network-sercvice-policy-list."""
        resources = 'network_service_policies'
        cmd = gbp.ListNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                           None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_network_sercice_policies_sort(self):
        """network-service-policy-list --sort-key name --sort-key id
        --sort-key asc --sort-key desc
        """
        resources = 'network_service_policies'
        cmd = gbp.ListNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                           None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_network_service_polices_limit(self):
        """network-service-policy-list -P."""
        resources = 'network_service_policies'
        cmd = gbp.ListNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                           None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_network_service_policy_id(self):
        """network-service-policy-show test_id."""
        resource = 'network_service_policy'
        cmd = gbp.ShowNetworkServicePolicy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_network_service_policy_id_name(self):
        """network-service-policy-show."""
        resource = 'network_service_policy'
        cmd = gbp.ShowNetworkServicePolicy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_network_service_policy(self):
        """network-service-policy-update  myid --name myname --tags a b."""
        resource = 'network_service_policy'
        cmd = gbp.UpdateNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_network_service_policy_with_allparams(self):
        """network-service-policy-update with all params."""
        resource = 'network_service_policy'
        cmd = gbp.UpdateNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                             None)
        name = 'nsp'
        description = 'nsp description'
        my_id = 'someid'
        network_svc_params = "type=ip_single,name=vip,value=self_subnet"
        shared = 'true'
        args = [my_id,
                '--name', name,
                '--description', description,
                '--network-service-params', network_svc_params,
                '--shared', shared,
                '--request-format', 'json']
        params = {
            'name': name,
            'description': description,
            'network_service_params': [{"type": "ip_single", "name": "vip",
                                        "value": "self_subnet"}],
            'shared': shared
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_delete_network_service_policy(self):
        """network-service-policy-delete my-id."""
        resource = 'network_service_policy'
        cmd = gbp.DeleteNetworkServicePolicy(test_cli20.MyApp(sys.stdout),
                                             None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


import re


def str2dict(strdict):
    """Convert key1=value1,key2=value2,... string into dictionary.

    :param strdict: key1=value1,key2=value2
    Note: This implementation overrides the original implementation
    in the neutronclient such that it is no longer required to append
    the key with a = to specify a corresponding empty value. For example,
    key1=value1,key2,key3=value3
    key1
    key1,key2
    will also be supported and converted to a dictionary with empty
    values for the relevant keys.
    """
    if not strdict:
        return {}
    return dict([kv.split('=', 1) if '=' in kv else [kv, ""]
                 for kv in strdict.split(',')])


def str2list(strlist):
    """Convert key1,key2,... string into list.

    :param strlist: key1,key2
    strlist can be comma or space separated.
    """
    if strlist is not None:
        strlist = strlist.strip(', ')
    if not strlist:
        return []
    return re.split("[, ]+", strlist)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging
import sys

from gbpclient.gbp.v2_0 import groupbasedpolicy as gbp
from gbpclient.tests.unit import test_cli20


class CLITestV20L2PolicyJSON(test_cli20.CLITestV20Base):

    LOG = logging.getLogger(__name__)

    def setUp(self):
        super(CLITestV20L2PolicyJSON, self).setUp()

    def test_create_l2_policy_with_mandatory_params(self):
        resource = 'l2_policy'
        cmd = gbp.CreateL2Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_l2_policy_with_all_params(self):
        """l2-policy-create with all params."""
        resource = 'l2_policy'
        cmd = gbp.CreateL2Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        tenant_id = 'my-tenant'
        name = 'my-name'
        description = 'l2p description'
        l3_policy_id = 'l3p'
        inject_default_route = 'false'
        shared = 'true'
        args = [name,
                '--tenant-id', tenant_id,
                '--description', description,
                '--l3-policy-id', l3_policy_id,
                '--inject-default-route', inject_default_route,
                '--shared', shared]
        position_names = ['name', 'description', 'l3_policy_id']
        position_values = [name, description, l3_policy_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id, shared=shared,
                                   inject_default_route=inject_default_route)

    def test_create_l2_policy_with_reuse_bd(self):
        """l2-policy-create with reuse_bd option."""
        resource = 'l2_policy'
        cmd = gbp.CreateL2Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        tenant_id = 'my-tenant'
        name = 'my-name'
        target_l2p = 'another-l2p'
        args = [name,
                '--tenant-id', tenant_id,
                '--reuse-bd', target_l2p]
        position_names = ['name', 'reuse_bd']
        position_values = [name, target_l2p]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_list_l2_policies(self):
        resource = 'l2_policies'
        cmd = gbp.ListL2Policy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resource, cmd, True)

    def test_show_l2_policy(self):
        resource = 'l2_policy'
        cmd = gbp.ShowL2Policy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_l2_policy(self):
        resource = 'l2_policy'
        cmd = gbp.UpdateL2Policy(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_l2_policy_with_all_params(self):
        """l2-policy-update."""
        resource = 'l2_policy'
        cmd = gbp.UpdateL2Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'someid'
        name = 'l2policy'
        description = 'l2policy description'
        l3_policy_id = 'l3p'
        inject_default_route = 'false'
        shared = 'true'
        args = [my_id,
                '--name', name,
                '--description', description,
                '--l3-policy-id', l3_policy_id,
                '--inject-default-route', inject_default_route,
                '--shared', shared]
        params = {
            'name': name,
            'description': description,
            'l3_policy_id': l3_policy_id,
            'inject_default_route': inject_default_route,
            'shared': shared
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_delete_l2_policy_name(self):
        resource = 'l2_policy'
        cmd = gbp.DeleteL2Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging
import sys

from gbpclient.gbp.v2_0 import groupbasedpolicy as gbp
from gbpclient.tests.unit import test_cli20


class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):

    LOG = logging.getLogger(__name__)

    def setUp(self):
        super(CLITestV20PolicyTargetGroupJSON, self).setUp()

    def test_create_policy_target_group_with_mandatory_params(self):
        """policy-target-group-create with all mandatory params."""
        resource = 'policy_target_group'
        cmd = gbp.CreatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_policy_target_group_with_all_params(self):
        """policy-target-group-create with all params."""
        resource = 'policy_target_group'
        cmd = gbp.CreatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        tenant_id = 'my-tenant'
        name = 'my-name'
        description = 'ptg description'
        application_policy_group_id = 'application_policy_group_id'
        l2_policy_id = 'l2_policy_id'
        provided_prs = "icmp-prs=false,web-prs=true"
        consumed_prs = "ssh-prs=true,ftp-prs=false"
        network_service_policy_id = 'network_service_policy_id'
        shared = 'true'
        intra_ptg_allow = 'false'
        args = [name,
                '--tenant-id', tenant_id,
                '--description', description,
                '--application-policy-group-id', application_policy_group_id,
                '--l2-policy-id', l2_policy_id,
                '--provided-policy-rule-sets', provided_prs,
                '--consumed-policy-rule-sets', consumed_prs,
                '--network-service-policy-id', network_service_policy_id,
                '--shared', shared,
                '--intra-ptg-allow', intra_ptg_allow]
        position_names = ['name', 'description', 'application_policy_group_id',
                          'l2_policy_id',
                          'provided_policy_rule_sets',
                          'consumed_policy_rule_sets',
                          'network_service_policy_id']
        provided_policy_rule_sets = {
            'icmp-prs': 'false',
            'web-prs': 'true'}
        consumed_policy_rule_sets = {
            'ssh-prs': 'true',
            'ftp-prs': 'false'}
        position_values = [name, description, application_policy_group_id,
                           l2_policy_id,
                           provided_policy_rule_sets,
                           consumed_policy_rule_sets,
                           network_service_policy_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id, shared=shared,
                                   intra_ptg_allow=intra_ptg_allow)

    def test_list_policy_target_groups(self):
        """policy-target-group-list."""
        resource = 'policy_target_groups'
        cmd = gbp.ListPolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resource, cmd, True)

    def test_show_policy_target_group_name(self):
        """policy-target-group-show."""
        resource = 'policy_target_group'
        cmd = gbp.ShowPolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_policy_target_group_action(self):
        """policy-policy-action-update."""
        resource = 'policy_target_group'
        cmd = gbp.UpdatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_policy_target_group_with_all_params(self):
        """policy-target-group-update."""
        resource = 'policy_target_group'
        cmd = gbp.UpdatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'ptg'
        description = 'ptg description'
        application_policy_group_id = 'application_policy_group_id'
        l2_policy_id = 'l2_policy_id'
        provided_prs = "icmp-prs=false,web-prs=true"
        consumed_prs = "ssh-prs=true,ftp-prs=false"
        network_service_policy_id = 'network_service_policy_id'
        shared = 'true'
        intra_ptg_allow = 'false'
        args = [my_id,
                '--name', name,
                '--description', description,
                '--application-policy-group-id', application_policy_group_id,
                '--l2-policy-id', l2_policy_id,
                '--provided-policy-rule-sets', provided_prs,
                '--consumed-policy-rule-sets', consumed_prs,
                '--network-service-policy-id', network_service_policy_id,
                '--shared', shared,
                '--intra-ptg-allow', intra_ptg_allow]
        provided_policy_rule_sets = {
            'icmp-prs': 'false',
            'web-prs': 'true'}
        consumed_policy_rule_sets = {
            'ssh-prs': 'true',
            'ftp-prs': 'false'}
        params = {
            'name': name,
            'description': description,
            'application_policy_group_id': application_policy_group_id,
            'l2_policy_id': l2_policy_id,
            'provided_policy_rule_sets': provided_policy_rule_sets,
            'consumed_policy_rule_sets': consumed_policy_rule_sets,
            'network_service_policy_id': network_service_policy_id,
            'shared': shared,
            'intra_ptg_allow': intra_ptg_allow
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_policy_target_group_unset_prs(self):
        """policy-target-group-update."""
        resource = 'policy_target_group'
        cmd = gbp.UpdatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        provided_prs = ""
        consumed_prs = ""
        args = [my_id,
                '--provided-policy-rule-sets', provided_prs,
                '--consumed-policy-rule-sets', consumed_prs
                ]
        params = {
            'provided_policy_rule_sets': {},
            'consumed_policy_rule_sets': {}
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_policy_target_group_unset_nsp(self):
        """policy-target-group-update."""
        resource = 'policy_target_group'
        cmd = gbp.UpdatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        network_service_policy_id = ''
        args = [my_id,
                '--network-service-policy', network_service_policy_id]
        params = {'network_service_policy_id': None}
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_policy_target_group_unset_apg(self):
        """policy-target-group-update."""
        resource = 'policy_target_group'
        cmd = gbp.UpdatePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        application_policy_group_id = ''
        args = [my_id,
                '--application-policy-group', application_policy_group_id]
        params = {'application_policy_group_id': None}
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_delete_policy_target_group_name(self):
        """policy-target-group-delete."""
        resource = 'policy_target_group'
        cmd = gbp.DeletePolicyTargetGroup(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from gbpclient.gbp.v2_0 import servicechain
from gbpclient.tests.unit import test_cli20


class CLITestV20ServiceProfileJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20ServiceProfileJSON, self).setUp()

    def test_create_service_profile_with_mandatory_params(self):
        """service-profile-create with all mandatory params."""
        resource = 'service_profile'
        cmd = servicechain.CreateServiceProfile(test_cli20.MyApp(sys.stdout),
                                                None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id, name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_service_profile_with_all_params(self):
        """service-profile-create with all params."""
        resource = 'service_profile'
        cmd = servicechain.CreateServiceProfile(test_cli20.MyApp(sys.stdout),
                                                None)
        name = 'my-name'
        description = 'My Service Profile'
        tenant_id = 'my-tenant'
        shared = 'true'
        vendor = 'vendor'
        insertion_mode = 'some mode'
        service_type = 'servicetype1'
        service_flavor = 'cherry-garcia'
        my_id = 'my-id'
        args = ['--description', description,
                '--tenant-id', tenant_id,
                '--shared', shared,
                '--vendor', vendor,
                '--insertion-mode', insertion_mode,
                '--servicetype', service_type,
                '--service-flavor', service_flavor,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   description=description,
                                   tenant_id=tenant_id,
                                   shared=shared,
                                   vendor=vendor,
                                   insertion_mode=insertion_mode,
                                   service_type=service_type,
                                   service_flavor=service_flavor)

    def test_list_service_profiles(self):
        """service-profile-list."""
        resources = 'service_profiles'
        cmd = servicechain.ListServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, True)

    def test_list_service_profiles_pagination(self):
        """service-profile-list."""
        resources = 'service_profiles'
        cmd = servicechain.ListServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_service_profiles_sort(self):
        """service-profile-list --sort-key name --sort-key id --sort-key asc
        --sort-key desc
        """
        resources = 'service_profiles'
        cmd = servicechain.ListServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_service_profiles_limit(self):
        """service-profile-list -P."""
        resources = 'service_profiles'
        cmd = servicechain.ListServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_service_profile_id(self):
        """service-profile-show test_id."""
        resource = 'service_profile'
        cmd = servicechain.ShowServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_service_profile_id_name(self):
        """service-profile-show."""
        resource = 'service_profile'
        cmd = servicechain.ShowServiceProfile(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_service_profile(self):
        """service-profile-update  myid --name myname --tags a b."""
        resource = 'service_profile'
        cmd = servicechain.UpdateServiceProfile(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_service_profile_with_all_params(self):
        resource = 'service_profile'
        cmd = servicechain.UpdateServiceProfile(test_cli20.MyApp(sys.stdout),
                                                None)
        name = 'new-name'
        description = 'My Updated Service Profile'
        shared = 'true'
        vendor = 'open-source'
        insertion_mode = 'another mode'
        service_type = 'servicetype2'
        service_flavor = 'phish-food'
        body = {
            'name': name,
            'description': description,
            'shared': shared,
            'vendor': vendor,
            'insertion_mode': insertion_mode,
            'service_type': service_type,
            'service_flavor': service_flavor}
        args = ['myid', '--name', name,
                '--description', description,
                '--shared', shared,
                '--vendor', vendor,
                '--insertion-mode', insertion_mode,
                '--servicetype', service_type,
                '--service-flavor', service_flavor]
        self._test_update_resource(resource, cmd, 'myid', args, body)

    def test_delete_service_profile(self):
        """service-profile-delete my-id."""
        resource = 'service_profile'
        cmd = servicechain.DeleteServiceProfile(test_cli20.MyApp(sys.stdout),
                                                None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from gbpclient.gbp.v2_0 import groupbasedpolicy as gbp
from gbpclient.tests.unit import test_cli20


class CLITestV20PolicyRuleJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20PolicyRuleJSON, self).setUp()

    def test_create_policy_rule_with_mandatory_params(self):
        """grouppolicy-policy-rule-create with all mandatory params."""
        resource = 'policy_rule'
        cmd = gbp.CreatePolicyRule(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_policy_rule_with_all_params(self):
        """grouppolicy-policy-rule-create with all params."""
        resource = 'policy_rule'
        cmd = gbp.CreatePolicyRule(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        description = 'My PolicyRule'
        my_id = 'my-id'
        enabled = "true"
        policy_classifier_id = 'pc-id'
        policy_actions_res = ["pa1", "pa2"]
        policy_actions_arg = "pa1,pa2"
        shared = 'true'
        args = ['--tenant-id', tenant_id,
                '--description', description,
                '--enabled', enabled,
                '--classifier', policy_classifier_id,
                '--actions', policy_actions_arg,
                '--shared', shared,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id,
                                   description=description,
                                   enabled=enabled,
                                   policy_classifier_id=policy_classifier_id,
                                   policy_actions=policy_actions_res,
                                   shared=shared)

    def test_list_policy_rules(self):
        """grouppolicy-policy-rule-list."""
        resources = 'policy_rules'
        cmd = gbp.ListPolicyRule(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_policy_rules_pagination(self):
        """grouppolicy-policy-rule-list."""
        resources = 'policy_rules'
        cmd = gbp.ListPolicyRule(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_policy_rules_sort(self):
        """grouppolicy-policy-rule-list --sort-key name --sort-key id
        --sort-key asc --sort-key desc
        """
        resources = 'policy_rules'
        cmd = gbp.ListPolicyRule(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_policy_rules_limit(self):
        """grouppolicy-policy-rule-list -P."""
        resources = 'policy_rules'
        cmd = gbp.ListPolicyRule(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_policy_classifier_id(self):
        """grouppolicy-policy-rule-show test_id."""
        resource = 'policy_rule'
        cmd = gbp.ShowPolicyRule(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_policy_classifier_id_name(self):
        """grouppolicy-policy-rule-show."""
        resource = 'policy_rule'
        cmd = gbp.ShowPolicyRule(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_policy_rule(self):
        """grouppolicy-policy-rule-update  myid --name myname --tags a b."""
        resource = 'policy_rule'
        cmd = gbp.UpdatePolicyRule(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_policy_rule_with_allparams(self):
        resource = 'policy_rule'
        enabled = "true"
        policy_classifier_id = 'pc-id'
        policy_actions_res = ["pa1", "pa2"]
        policy_actions_arg = "pa1,pa2"
        my_id = 'someid'
        shared = 'true'
        cmd = gbp.UpdatePolicyRule(test_cli20.MyApp(sys.stdout), None)
        body = {
            'policy_classifier_id': policy_classifier_id,
            'enabled': enabled,
            'policy_actions': policy_actions_res,
            'shared': shared
        }
        args = [my_id,
                '--enabled', enabled,
                '--classifier', policy_classifier_id,
                '--actions', policy_actions_arg,
                '--shared', shared, ]
        self._test_update_resource(resource, cmd, my_id, args, body)

    def test_update_policy_rule_unset_actions(self):
        resource = 'policy_rule'
        policy_actions_res = []
        policy_actions_arg = ""
        my_id = 'someid'
        cmd = gbp.UpdatePolicyRule(test_cli20.MyApp(sys.stdout), None)
        body = {'policy_actions': policy_actions_res}
        args = [my_id, '--actions', policy_actions_arg]
        self._test_update_resource(resource, cmd, my_id, args, body)

    def test_delete_policy_classifier(self):
        """grouppolicy-policy-rule-delete my-id."""
        resource = 'policy_rule'
        cmd = gbp.DeletePolicyRule(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging

from gbpclient.gbp.v2_0 import purge as gbpclient_purge
from neutronclient.common import exceptions
from neutronclient.v2_0 import client as clientV2_0


_logger = logging.getLogger(__name__)


def exception_handler_v20(status_code, error_content):
    """Exception handler for API v2.0 client

        This routine generates the appropriate
        Neutron exception according to the contents of the
        response body

        :param status_code: HTTP error status code
        :param error_content: deserialized body of error response
    """
    error_dict = None
    if isinstance(error_content, dict):
        error_dict = error_content.get('NeutronError')
    # Find real error type
    bad_neutron_error_flag = False
    if error_dict:
        # If Neutron key is found, it will definitely contain
        # a 'message' and 'type' keys?
        try:
            error_type = error_dict['type']
            error_message = error_dict['message']
            if error_dict['detail']:
                error_message += "\n" + error_dict['detail']
        except Exception:
            bad_neutron_error_flag = True
        if not bad_neutron_error_flag:
            # If corresponding exception is defined, use it.
            client_exc = getattr(exceptions, '%sClient' % error_type, None)
            # Otherwise look up per status-code client exception
            if not client_exc:
                client_exc = exceptions.HTTP_EXCEPTION_MAP.get(status_code)
            if client_exc:
                raise client_exc(message=error_message,
                                 status_code=status_code)
            else:
                raise exceptions.NeutronClientException(
                    status_code=status_code, message=error_message)
        else:
            raise exceptions.NeutronClientException(status_code=status_code,
                                                    message=error_dict)
    else:
        message = None
        if isinstance(error_content, dict):
            message = error_content.get('message')
        if message:
            raise exceptions.NeutronClientException(status_code=status_code,
                                                    message=message)

    # If we end up here the exception was not a neutron error
    msg = "%s-%s" % (status_code, error_content)
    raise exceptions.NeutronClientException(status_code=status_code,
                                            message=msg)


class Client(clientV2_0.Client):
    """Client for the GBP API.

    :param string username: Username for authentication. (optional)
    :param string user_id: User ID for authentication. (optional)
    :param string password: Password for authentication. (optional)
    :param string token: Token for authentication. (optional)
    :param string tenant_name: Tenant name. (optional)
    :param string tenant_id: Tenant id. (optional)
    :param string auth_url: Keystone service endpoint for authorization.
    :param string service_type: Network service type to pull from the
                                keystone catalog (e.g. 'network') (optional)
    :param string endpoint_type: Network service endpoint type to pull from the
                                 keystone catalog (e.g. 'publicURL',
                                 'internalURL', or 'adminURL') (optional)
    :param string region_name: Name of a region to select when choosing an
                               endpoint from the service catalog.
    :param string endpoint_url: A user-supplied endpoint URL for the neutron
                            service.  Lazy-authentication is possible for API
                            service calls if endpoint is set at
                            instantiation.(optional)
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    :param bool insecure: SSL certificate validation. (optional)
    :param string ca_cert: SSL CA bundle file to use. (optional)
    :param integer retries: How many times idempotent (GET, PUT, DELETE)
                            requests to Neutron server should be retried if
                            they fail (default: 0).
    :param bool raise_errors: If True then exceptions caused by connection
                              failure are propagated to the caller.
                              (default: True)
    :param session: Keystone client auth session to use. (optional)
    :param auth: Keystone auth plugin to use. (optional)

    Example::

        from gbpclient.v2_0 import client
        gbp = client.Client(username=USER,
                            password=PASS,
                            tenant_name=TENANT_NAME,
                            auth_url=KEYSTONE_URL)

        ptgs = gbp.list_policy_target_groups()
        ...

    """

    policy_targets_path = "/grouppolicy/policy_targets"
    policy_target_path = "/grouppolicy/policy_targets/%s"
    policy_target_groups_path = "/grouppolicy/policy_target_groups"
    policy_target_group_path = "/grouppolicy/policy_target_groups/%s"
    application_policy_groups_path = "/grouppolicy/application_policy_groups"
    application_policy_group_path = "/grouppolicy/application_policy_groups/%s"
    l2_policies_path = "/grouppolicy/l2_policies"
    l2_policy_path = "/grouppolicy/l2_policies/%s"
    l3_policies_path = "/grouppolicy/l3_policies"
    l3_policy_path = "/grouppolicy/l3_policies/%s"
    network_service_policies_path = "/grouppolicy/network_service_policies"
    network_service_policy_path = "/grouppolicy/network_service_policies/%s"
    external_policies_path = "/grouppolicy/external_policies"
    external_policy_path = "/grouppolicy/external_policies/%s"
    external_segments_path = "/grouppolicy/external_segments"
    external_segment_path = "/grouppolicy/external_segments/%s"
    nat_pools_path = "/grouppolicy/nat_pools"
    nat_pool_path = "/grouppolicy/nat_pools/%s"
    policy_classifiers_path = "/grouppolicy/policy_classifiers"
    policy_classifier_path = "/grouppolicy/policy_classifiers/%s"
    policy_actions_path = "/grouppolicy/policy_actions"
    policy_action_path = "/grouppolicy/policy_actions/%s"
    policy_rules_path = "/grouppolicy/policy_rules"
    policy_rule_path = "/grouppolicy/policy_rules/%s"
    policy_rule_sets_path = "/grouppolicy/policy_rule_sets"
    policy_rule_set_path = "/grouppolicy/policy_rule_sets/%s"
    service_profiles_path = "/servicechain/service_profiles"
    service_profile_path = "/servicechain/service_profiles/%s"
    servicechain_nodes_path = "/servicechain/servicechain_nodes"
    servicechain_node_path = "/servicechain/servicechain_nodes/%s"
    servicechain_specs_path = "/servicechain/servicechain_specs"
    servicechain_spec_path = "/servicechain/servicechain_specs/%s"
    servicechain_instances_path = "/servicechain/servicechain_instances"
    servicechain_instance_path = "/servicechain/servicechain_instances/%s"

    # API has no way to report plurals, so we have to hard code them
    EXTED_PLURALS = {'policy_targets': 'policy_target',
                     'policy_target_groups': 'policy_target_group',
                     'application_policy_groups': 'application_policy_group',
                     'l2_policies': 'l2_policy',
                     'l3_policies': 'l3_policy',
                     'network_service_policies': 'network_service_policy',
                     'external_policies': 'external_policy',
                     'external_segments': 'external_segment',
                     'nat_pools': 'nat_pool',
                     'policy_classifiers': 'policy_classifier',
                     'policy_actions': 'policy_action',
                     'policy_rules': 'policy_rule',
                     'policy_rule_sets': 'policy_rule_set',
                     }
    # 8192 Is the default max URI len for eventlet.wsgi.server
    MAX_URI_LEN = 8192

    def list_extensions(self, **_params):
        """Fetch a list of all exts on server side."""
        return self.get(self.extensions_path, params=_params)

    def show_extension(self, ext_alias, **_params):
        """Fetch a list of all exts on server side."""
        return self.get(self.extension_path % ext_alias, params=_params)

    def list_policy_targets(self, retrieve_all=True, **_params):
        """Fetches a list of all policy targets for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_targets', self.policy_targets_path,
                         retrieve_all, **_params)

    def show_policy_target(self, policy_target, **_params):
        """Fetches information of a certain policy target."""
        return self.get(self.policy_target_path % (policy_target),
                        params=_params)

    def list_application_policy_groups(self, retrieve_all=True, **_params):
        """Fetches a list of all application_policy_groups for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('application_policy_groups',
                         self.application_policy_groups_path,
                         retrieve_all, **_params)

    def show_application_policy_group(
        self, application_policy_group, **_params):
        """Fetches information of a certain application_policy_group."""
        return self.get(self.application_policy_group_path % (
            application_policy_group), params=_params)

    def create_application_policy_group(self, body=None):
        """Creates a new application_policy_group."""
        return self.post(self.application_policy_groups_path, body=body)

    def update_application_policy_group(
        self, application_policy_group, body=None):
        """Updates a application_policy_group."""
        return self.put(
            self.application_policy_group_path % (application_policy_group),
            body=body)

    def delete_application_policy_group(self, application_policy_group):
        """Deletes the specified application_policy_group."""
        return self.delete(
            self.application_policy_group_path % (application_policy_group))

    def create_policy_target(self, body=None):
        """Creates a new policy target."""
        return self.post(self.policy_targets_path, body=body)

    def update_policy_target(self, policy_target, body=None):
        """Updates a policy target."""
        return self.put(self.policy_target_path % (policy_target), body=body)

    def delete_policy_target(self, policy_target):
        """Deletes the specified policy target."""
        return self.delete(self.policy_target_path % (policy_target))

    def list_policy_target_groups(self, retrieve_all=True, **_params):
        """Fetches a list of all policy target_groups for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_target_groups',
                         self.policy_target_groups_path, retrieve_all,
                         **_params)

    def show_policy_target_group(self, policy_target_group, **_params):
        """Fetches information of a certain policy target_group."""
        return self.get(self.policy_target_group_path % (policy_target_group),
                        params=_params)

    def create_policy_target_group(self, body=None):
        """Creates a new policy target_group."""
        return self.post(self.policy_target_groups_path, body=body)

    def update_policy_target_group(self, policy_target_group, body=None):
        """Updates a policy target_group."""
        return self.put(self.policy_target_group_path % (policy_target_group),
                        body=body)

    def delete_policy_target_group(self, policy_target_group):
        """Deletes the specified policy target_group."""
        return self.delete(
            self.policy_target_group_path % (policy_target_group))

    def list_l2_policies(self, retrieve_all=True, **_params):
        """Fetches a list of all l2_policies for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('l2_policies', self.l2_policies_path,
                         retrieve_all, **_params)

    def show_l2_policy(self, l2_policy, **_params):
        """Fetches information of a certain l2_policy."""
        return self.get(self.l2_policy_path % (l2_policy),
                        params=_params)

    def create_l2_policy(self, body=None):
        """Creates a new l2_policy."""
        return self.post(self.l2_policies_path, body=body)

    def update_l2_policy(self, l2_policy, body=None):
        """Updates a l2_policy."""
        return self.put(self.l2_policy_path % (l2_policy), body=body)

    def delete_l2_policy(self, l2_policy):
        """Deletes the specified l2_policy."""
        return self.delete(self.l2_policy_path % (l2_policy))

    def list_network_service_policies(self, retrieve_all=True, **_params):
        """Fetches a list of all network_service_policies for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('network_service_policies',
                         self.network_service_policies_path,
                         retrieve_all, **_params)

    def show_network_service_policy(self, network_service_policy, **_params):
        """Fetches information of a certain network_service_policy."""
        return self.get(
            self.network_service_policy_path % (network_service_policy),
            params=_params)

    def create_network_service_policy(self, body=None):
        """Creates a new network_service_policy."""
        return self.post(self.network_service_policies_path, body=body)

    def update_network_service_policy(self, network_service_policy, body=None):
        """Updates a network_service_policy."""
        return self.put(
            self.network_service_policy_path % (network_service_policy),
            body=body)

    def delete_network_service_policy(self, network_service_policy):
        """Deletes the specified network_service_policy."""
        return self.delete(
            self.network_service_policy_path % (network_service_policy))

    def list_external_policies(self, retrieve_all=True, **_params):
        """Fetches a list of all external_policies for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('external_policies',
                         self.external_policies_path,
                         retrieve_all, **_params)

    def show_external_policy(self, external_policy, **_params):
        """Fetches information of a certain external_policy."""
        return self.get(
            self.external_policy_path % (external_policy),
            params=_params)

    def create_external_policy(self, body=None):
        """Creates a new external_policy."""
        return self.post(self.external_policies_path, body=body)

    def update_external_policy(self, external_policy, body=None):
        """Updates a external_policy."""
        return self.put(
            self.external_policy_path % (external_policy),
            body=body)

    def delete_external_policy(self, external_policy):
        """Deletes the specified external_policy."""
        return self.delete(
            self.external_policy_path % (external_policy))

    def list_external_segments(self, retrieve_all=True, **_params):
        """Fetches a list of all external_segments for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('external_segments',
                         self.external_segments_path,
                         retrieve_all, **_params)

    def show_external_segment(self, external_segment, **_params):
        """Fetches information of a certain external_segment."""
        return self.get(
            self.external_segment_path % (external_segment),
            params=_params)

    def create_external_segment(self, body=None):
        """Creates a new external_segment."""
        return self.post(self.external_segments_path, body=body)

    def update_external_segment(self, external_segment, body=None):
        """Updates a external_segment."""
        return self.put(
            self.external_segment_path % (external_segment),
            body=body)

    def delete_external_segment(self, external_segment):
        """Deletes the specified external_segment."""
        return self.delete(
            self.external_segment_path % (external_segment))

    def list_nat_pools(self, retrieve_all=True, **_params):
        """Fetches a list of all nat_pools for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('nat_pools',
                         self.nat_pools_path,
                         retrieve_all, **_params)

    def show_nat_pool(self, nat_pool, **_params):
        """Fetches information of a certain nat_pool."""
        return self.get(self.nat_pool_path % (nat_pool), params=_params)

    def create_nat_pool(self, body=None):
        """Creates a new nat_pool."""
        return self.post(self.nat_pools_path, body=body)

    def update_nat_pool(self, nat_pool, body=None):
        """Updates a nat_pool."""
        return self.put(self.nat_pool_path % (nat_pool), body=body)

    def delete_nat_pool(self, nat_pool):
        """Deletes the specified nat_pool."""
        return self.delete(self.nat_pool_path % (nat_pool))

    def list_l3_policies(self, retrieve_all=True, **_params):
        """Fetches a list of all l3_policies for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('l3_policies', self.l3_policies_path,
                         retrieve_all, **_params)

    def show_l3_policy(self, l3_policy, **_params):
        """Fetches information of a certain l3_policy."""
        return self.get(self.l3_policy_path % (l3_policy),
                        params=_params)

    def create_l3_policy(self, body=None):
        """Creates a new l3_policy."""
        return self.post(self.l3_policies_path, body=body)

    def update_l3_policy(self, l3_policy, body=None):
        """Updates a l3_policy."""
        return self.put(self.l3_policy_path % (l3_policy),
                        body=body)

    def delete_l3_policy(self, l3_policy):
        """Deletes the specified l3_policy."""
        return self.delete(self.l3_policy_path % (l3_policy))

    def list_policy_classifiers(self, retrieve_all=True, **_params):
        """Fetches a list of all policy_classifiers for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_classifiers', self.policy_classifiers_path,
                         retrieve_all, **_params)

    def show_policy_classifier(self, policy_classifier, **_params):
        """Fetches information of a certain policy_classifier."""
        return self.get(self.policy_classifier_path % (policy_classifier),
                        params=_params)

    def create_policy_classifier(self, body=None):
        """Creates a new policy_classifier."""
        return self.post(self.policy_classifiers_path, body=body)

    def update_policy_classifier(self, policy_classifier, body=None):
        """Updates a policy_classifier."""
        return self.put(self.policy_classifier_path % (policy_classifier),
                        body=body)

    def delete_policy_classifier(self, policy_classifier):
        """Deletes the specified policy_classifier."""
        return self.delete(self.policy_classifier_path % (policy_classifier))

    def list_policy_actions(self, retrieve_all=True, **_params):
        """Fetches a list of all policy_actions for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_actions', self.policy_actions_path,
                         retrieve_all, **_params)

    def show_policy_action(self, policy_action, **_params):
        """Fetches information of a certain policy_action."""
        return self.get(self.policy_action_path % (policy_action),
                        params=_params)

    def create_policy_action(self, body=None):
        """Creates a new policy_action."""
        return self.post(self.policy_actions_path, body=body)

    def update_policy_action(self, policy_action, body=None):
        """Updates a policy_action."""
        return self.put(self.policy_action_path % (policy_action), body=body)

    def delete_policy_action(self, policy_action):
        """Deletes the specified policy_action."""
        return self.delete(self.policy_action_path % (policy_action))

    def list_policy_rules(self, retrieve_all=True, **_params):
        """Fetches a list of all policy_rules for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_rules', self.policy_rules_path, retrieve_all,
                         **_params)

    def show_policy_rule(self, policy_rule, **_params):
        """Fetches information of a certain policy_rule."""
        return self.get(self.policy_rule_path % (policy_rule), params=_params)

    def create_policy_rule(self, body=None):
        """Creates a new policy_rule."""
        return self.post(self.policy_rules_path, body=body)

    def update_policy_rule(self, policy_rule, body=None):
        """Updates a policy_rule."""
        return self.put(self.policy_rule_path % (policy_rule), body=body)

    def delete_policy_rule(self, policy_rule):
        """Deletes the specified policy_rule."""
        return self.delete(self.policy_rule_path % (policy_rule))

    def list_policy_rule_sets(self, retrieve_all=True, **_params):
        """Fetches a list of all Policy Rule Sets for a tenant."""
        # Pass filters in "params" argument to do_request
        return self.list('policy_rule_sets', self.policy_rule_sets_path,
                         retrieve_all, **_params)

    def show_policy_rule_set(self, policy_rule_set, **_params):
        """Fetches information of a certain Policy Rule Set."""
        return self.get(self.policy_rule_set_path % (policy_rule_set),
                        params=_params)

    def create_policy_rule_set(self, body=None):
        """Creates a new Policy Rule Set."""
        return self.post(self.policy_rule_sets_path, body=body)

    def update_policy_rule_set(self, policy_rule_set, body=None):
        """Updates a Policy Rule Set."""
        return self.put(self.policy_rule_set_path % (policy_rule_set),
                        body=body)

    def delete_policy_rule_set(self, policy_rule_set):
        """Deletes the specified Policy Rule Set."""
        return self.delete(self.policy_rule_set_path % (policy_rule_set))

    def list_service_profiles(self, retrieve_all=True, **_params):

        """Fetches a list of all service profiles for a tenant."""
        # Pass filters in "params" argument to do_request

        return self.list('service_profiles', self.service_profiles_path,
                         retrieve_all, **_params)

    def show_service_profile(self, service_profile, **_params):
        """Fetches information of a certain service profile."""
        return self.get(self.service_profile_path % (service_profile),
                        params=_params)

    def create_service_profile(self, body=None):
        """Creates a new service profile."""
        return self.post(self.service_profiles_path, body=body)

    def update_service_profile(self, service_profile, body=None):
        """Updates a service profile."""
        return self.put(self.service_profile_path % (service_profile),
                        body=body)

    def delete_service_profile(self, service_profile):
        """Deletes the specified service profile."""
        return self.delete(self.service_profile_path % (service_profile))

    def list_servicechain_nodes(self, retrieve_all=True, **_params):

        """Fetches a list of all service chain nodes for a tenant."""
        # Pass filters in "params" argument to do_request

        return self.list('servicechain_nodes', self.servicechain_nodes_path,
                         retrieve_all, **_params)

    def show_servicechain_node(self, servicechain_node, **_params):
        """Fetches information of a certain service chain node."""
        return self.get(self.servicechain_node_path % (servicechain_node),
                        params=_params)

    def create_servicechain_node(self, body=None):
        """Creates a new service chain node."""
        return self.post(self.servicechain_nodes_path, body=body)

    def update_servicechain_node(self, servicechain_node, body=None):
        """Updates a service chain node."""
        return self.put(self.servicechain_node_path % (servicechain_node),
                        body=body)

    def delete_servicechain_node(self, servicechain_node):
        """Deletes the specified service chain node."""
        return self.delete(self.servicechain_node_path % (servicechain_node))

    def list_servicechain_specs(self, retrieve_all=True, **_params):
        """Fetches a list of all service chain specs for a tenant."""
        # Pass filters in "params" argument to do_request

        return self.list('servicechain_specs', self.servicechain_specs_path,
                         retrieve_all, **_params)

    def show_servicechain_spec(self, servicechain_spec, **_params):
        """Fetches information of a certain service chain spec."""
        return self.get(self.servicechain_spec_path % (servicechain_spec),
                        params=_params)

    def create_servicechain_spec(self, body=None):
        """Creates a new service chain spec."""
        return self.post(self.servicechain_specs_path, body=body)

    def update_servicechain_spec(self, servicechain_spec, body=None):
        """Updates a service chain spec."""
        return self.put(self.servicechain_spec_path % (servicechain_spec),
                        body=body)

    def delete_servicechain_spec(self, servicechain_spec):
        """Deletes the specified service chain spec."""
        return self.delete(self.servicechain_spec_path % (servicechain_spec))

    def list_servicechain_instances(self, retrieve_all=True, **_params):
        """Fetches a list of all service chain instances for a tenant."""
        # Pass filters in "params" argument to do_request

        return self.list('servicechain_instances',
                         self.servicechain_instances_path,
                         retrieve_all, **_params)

    def show_servicechain_instance(self, servicechain_instance, **_params):
        """Fetches information of a certain service chain instance."""
        return self.get(self.servicechain_instance_path %
                        (servicechain_instance), params=_params)

    def create_servicechain_instance(self, body=None):
        """Creates a new service chain instance."""
        return self.post(self.servicechain_instances_path, body=body)

    def update_servicechain_instance(self, servicechain_instance, body=None):
        """Updates a service chain instance."""
        return self.put(self.servicechain_instance_path %
                        (servicechain_instance), body=body)

    def delete_servicechain_instance(self, servicechain_instance):
        """Deletes the specified service chain instance."""
        return self.delete(self.servicechain_instance_path %
                           (servicechain_instance))

    def purge(self, tenant_id):
        purge_obj = gbpclient_purge.PurgeAPI(None, None, self)
        purge_obj.take_action(tenant_id)

    def __init__(self, **kwargs):
        """Initialize a new client for the GBP v2.0 API."""
        super(Client, self).__init__(**kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient.tests.unit import test_cli20_purge


class CLITestV20Purge(test_cli20_purge.CLITestV20Purge):
    def setUp(self):
        super(CLITestV20Purge, self).setUp()
        self.resource_types = ['policy_target', 'policy_target_group',
                               'l2_policy', 'l3_policy', 'external_policy',
                               'nat_pool', 'external_segment',
                               'policy_rule_set', 'policy_rule',
                               'policy_classifier', 'policy_action',
                               'network_service_policy',
                               'servicechain_instance', 'servicechain_spec',
                               'servicechain_node', 'service_profile',
                               'application_policy_group']



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse
import logging

from neutronclient._i18n import _
from neutronclient.common import utils as n_utils
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.v2_0 import client as neutron_client
from oslo_serialization import jsonutils

from gbpclient.common import utils

neutron_client.UUID_PATTERN = neutron_client.UUID_PATTERN + (
    '|auto' + neutron_client.HEX_ELEM + '{32}')


def _format_fixed_ips(pt):
    try:
        return '\n'.join([jsonutils.dumps(ip) for ip in pt['fixed_ips']])
    except (TypeError, KeyError):
        return ''


def _format_network_service_params(net_svc_policy):
    try:
        return '\n'.join([jsonutils.dumps(param) for param in
                          net_svc_policy['network_service_params']])
    except (TypeError, KeyError):
        return ''


def _format_host_routes(subnet):
    try:
        return '\n'.join([jsonutils.dumps(route) for route in
                          subnet['host_routes']])
    except (TypeError, KeyError):
        return ''


class ListPolicyTarget(neutronV20.ListCommand):
    """List policy_targets that belong to a given tenant."""

    resource = 'policy_target'
    log = logging.getLogger(__name__ + '.ListPolicyTarget')
    _formatters = {'fixed_ips': _format_fixed_ips, }
    list_columns = ['id', 'name', 'description', 'policy_target_group_id',
                    'port_id', 'fixed_ips']
    pagination_support = True
    sorting_support = True


class ShowPolicyTarget(neutronV20.ShowCommand):
    """Show information of a given policy_target."""

    resource = 'policy_target'
    log = logging.getLogger(__name__ + '.ShowPolicyTarget')


class CreatePolicyTarget(neutronV20.CreateCommand):
    """Create a policy_target for a given tenant."""

    resource = 'policy_target'
    log = logging.getLogger(__name__ + '.CreatePolicyTarget')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the Policy Target'))
        parser.add_argument(
            '--policy-target-group',
            help=_('Policy Target Group (required argument)'))
        parser.add_argument(
            '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
            action='append',
            type=n_utils.str2dict_type(optional_keys=['subnet_id',
                                                      'ip_address']),
            help=_('Desired IP and/or subnet for this Policy Target: '
                   'subnet_id=<nid>,ip_address=<ip>. '
                   'You can repeat this option.'))
        parser.add_argument(
            '--fixed_ip', action='append', help=argparse.SUPPRESS)
        parser.add_argument(
            '--port-id', default='',
            help=_('Neutron Port UUID'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of Policy Target to create (required argument)'))
        parser.add_argument(
            '--segmentation-labels', type=utils.str2list,
            help=_('Comma separated list of segmentation labels, each label '
                   'can be upto 255 characters. This option is currently '
                   'only available with the APIC backend.'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'segmentation_labels'])
        if parsed_args.policy_target_group:
            body[self.resource]['policy_target_group_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_target_group',
                    parsed_args.policy_target_group)

        if parsed_args.port_id:
            body[self.resource]['port_id'] = (
                parsed_args.port_id)

        if parsed_args.fixed_ip:
            body[self.resource]['fixed_ips'] = parsed_args.fixed_ip

        return body


class DeletePolicyTarget(neutronV20.DeleteCommand):
    """Delete a given Policy Target."""

    resource = 'policy_target'
    log = logging.getLogger(__name__ + '.DeletePolicyTarget')


class UpdatePolicyTarget(neutronV20.UpdateCommand):
    """Update Policy Target's information."""

    resource = 'policy_target'
    log = logging.getLogger(__name__ + '.UpdatePolicyTarget')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the Policy Target'))
        parser.add_argument(
            '--name',
            help=_('New name of the Policy Target'))
        parser.add_argument(
            '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
            action='append',
            type=n_utils.str2dict_type(optional_keys=['subnet_id',
                                                      'ip_address']),
            help=_('Desired IP and/or subnet for this Policy Target: '
                   'subnet_id=<nid>,ip_address=<ip>. '
                   'You can repeat this option.'))
        parser.add_argument(
            '--fixed_ip', action='append', help=argparse.SUPPRESS)
        parser.add_argument(
            '--segmentation-labels', type=utils.str2list,
            help=_('Comma separated list of segmentation labels, each label '
                   'can be upto 255 characters. This option is currently '
                   'only available with the APIC backend.'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'segmentation_labels'])

        if parsed_args.fixed_ip:
            body[self.resource]['fixed_ips'] = parsed_args.fixed_ip

        return body


class ListPolicyTargetGroup(neutronV20.ListCommand):
    """List Policy Target Groups that belong to a given tenant."""

    resource = 'policy_target_group'
    log = logging.getLogger(__name__ + '.ListPolicyTargetGroup')
    list_columns = ['id', 'name', 'description', 'application_policy_group_id',
                    'l2_policy_id', 'subnets']
    pagination_support = True
    sorting_support = True


class ShowPolicyTargetGroup(neutronV20.ShowCommand):
    """Show information of a given Policy Target Group."""

    resource = 'policy_target_group'
    log = logging.getLogger(__name__ + '.ShowPolicyTargetGroup')


class CreatePolicyTargetGroup(neutronV20.CreateCommand):
    """Create a Policy Target Group for a given tenant."""

    resource = 'policy_target_group'
    log = logging.getLogger(__name__ + '.CreatePolicyTargetGroup')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the Policy Target Group'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of Policy Target Group to create '
                   '(required argument)'))
        parser.add_argument(
            '--application-policy-group', metavar='APPLICATION_POLICY_GROUP',
            default='',
            help=_('Application Policy Group UUID (optional, default '
                   'is None)'))
        parser.add_argument(
            '--l2-policy', metavar='L2_POLICY',
            default='',
            help=_('L2 Policy UUID (if not specified, default is used)'))
        parser.add_argument(
            '--provided-policy-rule-sets', type=utils.str2dict,
            help=_('Comma separated list of Policy Rule Sets'))
        parser.add_argument(
            '--consumed-policy-rule-sets', type=utils.str2dict,
            help=_('Comma separated list of Policy Rule Sets'))
        parser.add_argument(
            '--network-service-policy', metavar='NETWORK_SERVICE_POLICY',
            default='',
            help=_('Network Service Policy'))
        parser.add_argument(
            '--subnets', type=utils.str2list,
            help=_('Comma separated list of Neutron Subnet UUIDs'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))
        n_utils.add_boolean_argument(
            parser, '--intra-ptg-allow', dest='intra_ptg_allow',
            help=_("Allow or disallow communication between Policy Targets of "
                   "this Policy Target Group. Default is True, i.e., all "
                   "Policy Targets can communicate with each other. This "
                   "option is only available with the APIC backend."))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.l2_policy:
            body[self.resource]['l2_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'l2_policy',
                    parsed_args.l2_policy)

        if parsed_args.application_policy_group:
            body[self.resource]['application_policy_group_id'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'application_policy_group',
                    parsed_args.application_policy_group))

        if parsed_args.network_service_policy:
            body[self.resource]['network_service_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'network_service_policy',
                    parsed_args.network_service_policy)

        if parsed_args.provided_policy_rule_sets:
            for key in parsed_args.provided_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.provided_policy_rule_sets[id_key] = \
                    parsed_args.provided_policy_rule_sets.pop(key)

        if parsed_args.consumed_policy_rule_sets:
            for key in parsed_args.consumed_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.consumed_policy_rule_sets[id_key] = \
                    parsed_args.consumed_policy_rule_sets.pop(key)

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'provided_policy_rule_sets', 'subnets',
                                'consumed_policy_rule_sets', 'shared',
                                'intra_ptg_allow'])

        return body


class DeletePolicyTargetGroup(neutronV20.DeleteCommand):
    """Delete a given Policy Target Group."""

    resource = 'policy_target_group'
    log = logging.getLogger(__name__ + '.DeletePolicyTargetGroup')


class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
    """Update Policy Target Group's information."""

    resource = 'policy_target_group'
    log = logging.getLogger(__name__ + '.UpdatePolicyTargetGroup')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the Policy Target Group'))
        parser.add_argument(
            '--l2-policy', metavar='L2_POLICY',
            help=_('New L2 policy'))
        parser.add_argument(
            '--application-policy-group', metavar='APPLICATION_POLICY_GROUP',
            help=_('New Application Policy Group'))
        parser.add_argument(
            '--network-service-policy', metavar='NETWORK_SERVICE_POLICY',
            help=_('New Network Service Policy'))
        parser.add_argument(
            '--provided-policy-rule-sets', type=utils.str2dict,
            help=_('New comma separated list of Policy Rule Sets '
                   '(to unset use "")'))
        parser.add_argument(
            '--consumed-policy-rule-sets', type=utils.str2dict,
            help=_('New comma separated list of Policy Rule Sets '
                   '(to unset use "")'))
        parser.add_argument(
            '--subnets', type=utils.str2list,
            help=_('New comma separated list of Neutron Subnet UUIDs '
                   '(to unset use "")'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))
        n_utils.add_boolean_argument(
            parser, '--intra-ptg-allow', dest='intra_ptg_allow',
            help=_("Allow or disallow communication between Policy Targets of "
                   "this Policy Target Group. Default is True, i.e., all "
                   "Policy Targets can communicate with each other. This "
                   "option is only available with the APIC backend."))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.l2_policy:
            body[self.resource]['l2_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'l2_policy',
                    parsed_args.l2_policy)

        if parsed_args.application_policy_group == '':
            body[self.resource]['application_policy_group_id'] = None
        elif parsed_args.application_policy_group:
            body[self.resource]['application_policy_group_id'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'application_policy_group',
                    parsed_args.application_policy_group))

        if parsed_args.network_service_policy == '':
            body[self.resource]['network_service_policy_id'] = None
        elif parsed_args.network_service_policy:
            body[self.resource]['network_service_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'network_service_policy',
                    parsed_args.network_service_policy)

        if parsed_args.provided_policy_rule_sets:
            for key in parsed_args.provided_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.provided_policy_rule_sets[id_key] = \
                    parsed_args.provided_policy_rule_sets.pop(key)

        if parsed_args.consumed_policy_rule_sets:
            for key in parsed_args.consumed_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.consumed_policy_rule_sets[id_key] = \
                    parsed_args.consumed_policy_rule_sets.pop(key)

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'provided_policy_rule_sets', 'subnets',
                                'consumed_policy_rule_sets', 'shared',
                                'intra_ptg_allow'])

        return body


class ListL2Policy(neutronV20.ListCommand):
    """List L2 Policies that belong to a given tenant."""

    resource = 'l2_policy'
    log = logging.getLogger(__name__ + '.ListL2Policy')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'l3_policy_id', 'network_id',
                    'inject_default_route']
    pagination_support = True
    sorting_support = True


class ShowL2Policy(neutronV20.ShowCommand):
    """Show information of a given L2 Policy."""

    resource = 'l2_policy'
    log = logging.getLogger(__name__ + '.ShowL2Policy')


class CreateL2Policy(neutronV20.CreateCommand):
    """Create a L2 Policy for a given tenant."""

    resource = 'l2_policy'
    log = logging.getLogger(__name__ + '.CreateL2Policy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the L2 Policy'))
        parser.add_argument(
            '--network',
            help=_('Neutron Network UUID to map the L2 Policy to '
                   '(if not specified, new Neutron Network is created '
                   'implicitly)'))
        parser.add_argument(
            '--l3-policy',
            default='',
            help=_('L3 Policy UUID (if not specified default is used)'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of L2 Policy to create (required argument)'))
        n_utils.add_boolean_argument(
            parser, '--inject-default-route', dest='inject_default_route',
            help=_('Enable or disable injecting default route, '
                   'default is True'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))
        parser.add_argument(
            '--reuse-bd',
            default=None,
            help=_('Name or UUID of L2 Policy whose BridgeDomain should '
                   'be reused (APIC only)'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description', 'shared',
                                'inject_default_route'])
        if parsed_args.l3_policy:
            body[self.resource]['l3_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'l3_policy',
                    parsed_args.l3_policy)
        if parsed_args.network:
            body[self.resource]['network_id'] = (
                parsed_args.network)
        if parsed_args.reuse_bd:
            body[self.resource]['reuse_bd'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'l2_policy',
                    parsed_args.reuse_bd)
        return body


class DeleteL2Policy(neutronV20.DeleteCommand):
    """Delete a given L2 Policy."""

    resource = 'l2_policy'
    log = logging.getLogger(__name__ + '.DeleteL2Policy')


class UpdateL2Policy(neutronV20.UpdateCommand):
    """Update L2 Policy's information."""

    resource = 'l2_policy'
    log = logging.getLogger(__name__ + '.UpdateL2Policy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the L2 Policy'))
        parser.add_argument(
            '--l3-policy',
            default='',
            help=_('New L3 Policy'))
        parser.add_argument(
            '--name',
            help=_('New name of the L2 Policy'))
        n_utils.add_boolean_argument(
            parser, '--inject-default-route', dest='inject_default_route',
            help=_('Enable or disable injecting of default route'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description', 'shared',
                                'inject_default_route'])
        if parsed_args.l3_policy:
            body[self.resource]['l3_policy_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'l3_policy',
                    parsed_args.l3_policy)

        return body


class ListL3Policy(neutronV20.ListCommand):
    """List l3_policies that belong to a given tenant."""

    resource = 'l3_policy'
    log = logging.getLogger(__name__ + '.ListL3Policy')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'ip_pool',
                    'subnet_prefix_length']
    pagination_support = True
    sorting_support = True


class ShowL3Policy(neutronV20.ShowCommand):
    """Show information of a given L3 Policy."""

    resource = 'l3_policy'
    log = logging.getLogger(__name__ + '.ShowL3Policy')


class CreateL3Policy(neutronV20.CreateCommand):
    """Create a L3 Policy for a given tenant."""

    resource = 'l3_policy'
    log = logging.getLogger(__name__ + '.CreateL3Policy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the L3 Policy'))
        parser.add_argument(
            '--ip-version',
            type=int,
            # default=4, choices=[4, 6],
            help=_('IP version, default is 4'))
        parser.add_argument(
            '--ip-pool',
            help=_('CIDR of IP pool to create, default is 10.0.0.0/8'))
        parser.add_argument(
            '--subnet-prefix-length',
            type=int,
            help=_('Subnet prefix length, default is 24'))
        parser.add_argument(
            '--address-scope-v4-id',
            help=_('Neutron Address-scope v4 UUID '
                   '(if not specified, new Neutron Address-scope is '
                   'created implicitly based on ip_version)'))
        parser.add_argument(
            '--address-scope-v6-id',
            help=_('Neutron Address-scope v6 UUID '
                   '(if not specified, new Neutron Address-scope is '
                   'created implicitly based on ip_version)'))
        parser.add_argument(
            '--subnetpools-v4', type=utils.str2list,
            help=_('Comma separated list of Neutron Subnetpool v4 UUIDs '
                   'if ip_version and address scope is v4'))
        parser.add_argument(
            '--subnetpools-v6', type=utils.str2list,
            help=_('Comma separated list of Neutron Subnetpool v6 UUIDs '
                   'if ip_version and address scope is v6'))
        parser.add_argument(
            '--external-segment',
            action='append', dest='external_segments', type=utils.str2dict,
            # Note: The following format is also supported but we do not
            # show it to avoid confusion
            # help=_('Use format <ext-segment-id-1>=<ip-addr1:ipaddr2:...>'
            #        '(this option can be repeated)'))
            help=_('Comma separated list of External Segments'
                   '(this option can be repeated)'))
        parser.add_argument(
            '--routers', type=utils.str2list,
            help=_('Comma separated list of Neutron Router UUIDs'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of L3 policy to create (required argument)'))
        parser.add_argument(
            '--allowed-vm-names', type=utils.str2list,
            help=_('Comma separated list of allowed VM name regexes, each '
                   'regex can be up to 255 characters.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.external_segments:
            external_segments_dict = {}
            for external_segment in parsed_args.external_segments:
                external_segment_id = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'external_segment',
                    list(external_segment.keys())[0])
                ipaddrs = next(iter(external_segment.values()))
                if ipaddrs == "":
                    ipaddrs = []
                else:
                    ipaddrs = next(iter(external_segment.values())).split(':')
                external_segments_dict[external_segment_id] = ipaddrs

            body[self.resource]['external_segments'] = external_segments_dict

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'ip_pool',
                                'address_scope_v4_id', 'address_scope_v6_id',
                                'subnetpools_v4', 'subnetpools_v6',
                                'routers', 'subnet_prefix_length', 'shared',
                                'allowed_vm_names'])

        return body


class DeleteL3Policy(neutronV20.DeleteCommand):
    """Delete a given L3 Policy."""

    resource = 'l3_policy'
    log = logging.getLogger(__name__ + '.DeleteL3Policy')


class UpdateL3Policy(neutronV20.UpdateCommand):
    """Update L3 Policy's information."""

    resource = 'l3_policy'
    log = logging.getLogger(__name__ + '.UpdateL3Policy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the L3 Policy'))
        parser.add_argument(
            '--subnet-prefix-length',
            type=int,
            help=_('New subnet prefix length'))
        parser.add_argument(
            '--external-segment',
            action='append', dest='external_segments', type=utils.str2dict,
            # Note: The following format is also supported but we do not
            # show it to avoid confusion
            # help=_('Use format <ext-segment-id-1>=<ip-addr1:ipaddr2:...>'
            #        '(this option can be repeated)'))
            help=_('New comma separated list of External Segments'
                   '(this option can be repeated)'))
        parser.add_argument(
            '--subnetpools-v4', type=utils.str2list,
            help=_('New comma separated list of Neutron Subnetpool v4 UUIDs '
                   'if ip_version and address scope is v4'))
        parser.add_argument(
            '--subnetpools-v6', type=utils.str2list,
            help=_('New comma separated list of Neutron Subnetpool v6 UUIDs '
                   'if ip_version and address scope is v6'))
        parser.add_argument(
            '--routers', type=utils.str2list,
            help=_('New comma separated list of Neutron Router UUIDs'))
        parser.add_argument(
            '--name',
            help=_('New name of the L3 Policy'))
        parser.add_argument(
            '--allowed-vm-names', type=utils.str2list,
            help=_('Comma separated list of allowed VM name regexes, each '
                   'regex can be up to 255 characters.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.external_segments:
            external_segments_dict = {}
            for external_segment in parsed_args.external_segments:
                if not external_segment:
                    break
                external_segment_id = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'external_segment',
                    list(external_segment.keys())[0])
                ipaddrs = next(iter(external_segment.values()))
                if ipaddrs == "":
                    ipaddrs = []
                else:
                    ipaddrs = next(iter(external_segment.values())).split(':')
                external_segments_dict[external_segment_id] = ipaddrs

            body[self.resource]['external_segments'] = external_segments_dict

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'ip_pool', 'routers',
                                'subnetpools_v4', 'subnetpools_v6',
                                'subnet_prefix_length', 'shared',
                                'allowed_vm_names'])

        return body


class ListApplicationPolicyGroup(neutronV20.ListCommand):
    """List l3_policies that belong to a given tenant."""

    resource = 'application_policy_group'
    log = logging.getLogger(__name__ + '.ListApplicationPolicyGroup')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'shared']
    pagination_support = True
    sorting_support = True


class ShowApplicationPolicyGroup(neutronV20.ShowCommand):
    """Show information of a given Application Policy Group."""

    resource = 'application_policy_group'
    log = logging.getLogger(__name__ + '.ShowApplicationPolicyGroup')


class CreateApplicationPolicyGroup(neutronV20.CreateCommand):
    """Create a Application Policy Group for a given tenant."""

    resource = 'application_policy_group'
    log = logging.getLogger(__name__ + '.CreateApplicationPolicyGroup')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the Application Policy Group'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the Application Policy Group to create '
                   '(required argument)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'shared'])

        return body


class DeleteApplicationPolicyGroup(neutronV20.DeleteCommand):
    """Delete a given Application Policy Group."""

    resource = 'application_policy_group'
    log = logging.getLogger(__name__ + '.DeleteApplicationPolicyGroup')


class UpdateApplicationPolicyGroup(neutronV20.UpdateCommand):
    """Update Application Policy Group's information."""

    resource = 'application_policy_group'
    log = logging.getLogger(__name__ + '.UpdateApplicationPolicyGroup')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the Application Policy Group'))
        parser.add_argument(
            '--name',
            help=_('New name of the Application Policy Group'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'shared'])

        return body


class ListNetworkServicePolicy(neutronV20.ListCommand):
    """List Network Service Policies that belong to a given tenant."""

    resource = 'network_service_policy'
    log = logging.getLogger(__name__ + '.ListNetworkServicePolicy')
    _formatters = {'network_servie_params': _format_network_service_params}
    list_columns = ['id', 'name', 'description', 'network_service_params',
                    'policy_target_groups']
    pagination_support = True
    sorting_support = True


class ShowNetworkServicePolicy(neutronV20.ShowCommand):
    """Show information of a given network_service_policy."""

    resource = 'network_service_policy'
    log = logging.getLogger(__name__ + '.ShowNetworkServicePolicy')


class CreateNetworkServicePolicy(neutronV20.CreateCommand):
    """Create a Network Service Policy for a given tenant."""

    resource = 'network_service_policy'
    log = logging.getLogger(__name__ + '.CreateNetworkServicePolicy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the network_service_policy'))
        parser.add_argument(
            'name',
            help=_('Name of network_service_policy to create (required '
                   'argument)'))
        parser.add_argument(
            '--network-service-params',
            metavar='type=PARAM_TYPE,name=PARAM_NAME,value=PARAM_VALUE',
            action='append', dest='network_service_params',
            type=utils.str2dict,
            help=_('Params for this Network Service '
                   'Policy (this option can be repeated)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'network_service_params', 'shared'])
        return body


class DeleteNetworkServicePolicy(neutronV20.DeleteCommand):
    """Delete a given network_service_policy."""

    resource = 'network_service_policy'
    log = logging.getLogger(__name__ + '.DeleteNetworkServicePolicy')


class UpdateNetworkServicePolicy(neutronV20.UpdateCommand):
    """Update network_service_policy's information."""

    resource = 'network_service_policy'
    log = logging.getLogger(__name__ + '.UpdateNetworkServicePolicy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the network_service_policy'))
        parser.add_argument(
            '--name',
            help=_('New name of the network_service_policy'))
        parser.add_argument(
            '--network-service-params',
            metavar='type=PARAM_TYPE,name=PARAM_NAME,value=PARAM_VALUE',
            action='append', dest='network_service_params',
            type=utils.str2dict,
            help=_('New params for this Network Service '
                   'Policy (this option can be repeated)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'network_service_params', 'shared'])
        return body


class ListPolicyClassifier(neutronV20.ListCommand):
    """List classifiers that belong to a given tenant."""

    resource = 'policy_classifier'
    log = logging.getLogger(__name__ + '.ListPolicyClassifier')
    _formatters = {}
    list_columns = ['id', 'name', 'protocol', 'port_range', 'direction']
    pagination_support = True
    sorting_support = True


class ShowPolicyClassifier(neutronV20.ShowCommand):
    """Show information of a given classifier."""

    resource = 'policy_classifier'
    log = logging.getLogger(__name__ + '.ShowPolicyClassifier')


class CreatePolicyClassifier(neutronV20.CreateCommand):
    """Create a classifier for a given tenant."""

    resource = 'policy_classifier'
    log = logging.getLogger(__name__ + '.CreatePolicyClassifier')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('A description for the policy classifier'))
        parser.add_argument(
            '--protocol',
            help=_('A protocol value: tcp, udp, icmp or a protocol number '
                   'between 0 and 255. When not set, all protocols are '
                   'matched'))
        parser.add_argument(
            '--port-range',
            help=_('A port range value in the format p1[:p2]. '
                   'When not set all ports are matched'))
        parser.add_argument(
            '--direction',
            choices=['in', 'out', 'bi', ''],
            help=_('A direction value. When not set defaults to \'bi\''))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('A name for classifier to create (required argument)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'protocol', 'port_range',
                                'direction', 'shared'])

        if 'direction' not in body[self.resource]:
            body[self.resource]['direction'] = 'bi'

        return body


class DeletePolicyClassifier(neutronV20.DeleteCommand):
    """Delete a given classifier."""

    resource = 'policy_classifier'
    log = logging.getLogger(__name__ + '.DeletePolicyClassifier')


class UpdatePolicyClassifier(neutronV20.UpdateCommand):
    """Update classifier's information."""

    resource = 'policy_classifier'
    log = logging.getLogger(__name__ + '.UpdatePolicyClassifier')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description for the policy classifier'))
        parser.add_argument(
            '--protocol',
            help=_('New protocol value: tcp, udp, icmp or a protocol number '
                   'between 0 and 255 '
                   '(use empty string \'\' to unset and match all protocols)'))
        parser.add_argument(
            '--port-range',
            help=_('New port range value in the format p1[:p2] '
                   '(use empty string \'\' to unset)'))
        parser.add_argument(
            '--direction',
            choices=['in', 'out', 'bi', ''],
            help=_('New direction value'))
        parser.add_argument(
            '--name',
            help=_('New name of the classifier'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'protocol', 'port_range',
                                'direction', 'shared'])

        if parsed_args.protocol == '':
            body[self.resource]['protocol'] = None

        if parsed_args.port_range == '':
            body[self.resource]['port_range'] = None

        return body


class ListPolicyAction(neutronV20.ListCommand):
    """List actions that belong to a given tenant."""

    resource = 'policy_action'
    log = logging.getLogger(__name__ + '.ListPolicyAction')
    _formatters = {}
    list_columns = ['id', 'name', 'action_type', 'action_value']
    pagination_support = True
    sorting_support = True


class ShowPolicyAction(neutronV20.ShowCommand):
    """Show information of a given action."""

    resource = 'policy_action'
    log = logging.getLogger(__name__ + '.ShowPolicyAction')


class CreatePolicyAction(neutronV20.CreateCommand):
    """Create a action for a given tenant."""

    resource = 'policy_action'
    log = logging.getLogger(__name__ + '.CreatePolicyAction')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the policy action'))
        parser.add_argument(
            '--action-type',
            help=_('Type of action'))
        parser.add_argument(
            '--action-value',
            help=_('Name/UUID of servicechain spec for redirect action'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of action to create (required argument)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.action_value:
            body[self.resource]['action_value'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'servicechain_spec',
                    parsed_args.action_value))

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'action_type', 'shared'])

        return body


class DeletePolicyAction(neutronV20.DeleteCommand):
    """Delete a given action."""

    resource = 'policy_action'
    log = logging.getLogger(__name__ + '.DeletePolicyAction')


class UpdatePolicyAction(neutronV20.UpdateCommand):
    """Update action's information."""

    resource = 'policy_action'
    log = logging.getLogger(__name__ + '.UpdatePolicyAction')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the policy action'))
        parser.add_argument(
            '--action-value',
            help=_('New name/UUID of servicechain spec for redirect action'))
        parser.add_argument(
            '--name',
            help=_('New name of the action'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.action_value:
            body[self.resource]['action_value'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'servicechain_spec',
                    parsed_args.action_value))

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'shared'])

        return body


class ListPolicyRule(neutronV20.ListCommand):
    """List policy_rules that belong to a given tenant."""

    resource = 'policy_rule'
    log = logging.getLogger(__name__ + '.ListPolicyRule')
    _formatters = {}
    list_columns = ['id', 'name', 'enabled', 'classifier_id',
                    'actions']
    pagination_support = True
    sorting_support = True


class ShowPolicyRule(neutronV20.ShowCommand):
    """Show information of a given policy_rule."""

    resource = 'policy_rule'
    log = logging.getLogger(__name__ + '.ShowPolicyRule')


class CreatePolicyRule(neutronV20.CreateCommand):
    """Create a policy_rule for a given tenant."""

    resource = 'policy_rule'
    log = logging.getLogger(__name__ + '.CreatePolicyRule')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the policy_rule'))
        n_utils.add_boolean_argument(
            parser, '--enabled', dest='enabled',
            help=_('Enable flag, default is True '
                   '(if False, this Policy Rule is ignored)'))
        parser.add_argument(
            '--classifier',
            help=_('Policy Classifier (required argument)'))
        parser.add_argument(
            '--actions', type=utils.str2list,
            help=_('Comma separated list of Policy Action(s)'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of Policy Rule to create (required argument)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.actions:
            body[self.resource]['policy_actions'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_action',
                    elem) for elem in parsed_args.actions]

        if parsed_args.classifier:
            body[self.resource]['policy_classifier_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_classifier',
                    parsed_args.classifier)

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'enabled', 'shared'])

        return body


class DeletePolicyRule(neutronV20.DeleteCommand):
    """Delete a given policy_rule."""

    resource = 'policy_rule'
    log = logging.getLogger(__name__ + '.DeletePolicyRule')


class UpdatePolicyRule(neutronV20.UpdateCommand):
    """Update policy_rule's information."""

    resource = 'policy_rule'
    log = logging.getLogger(__name__ + '.UpdatePolicyRule')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('New name of the Policy Rule'))
        parser.add_argument(
            '--description',
            help=_('New description of the Policy Rule'))
        n_utils.add_boolean_argument(
            parser, '--enabled', dest='enabled',
            help=_('Enable flag (if False, this Policy Rule is ignored)'))
        parser.add_argument(
            '--classifier',
            help=_('New Policy Classifier'))
        parser.add_argument(
            '--actions', type=utils.str2list,
            help=_('New comma separated list of Policy Actions '
                   '(to unset use "")'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.actions == []:
            body[self.resource]['policy_actions'] = []
        elif parsed_args.actions:
            body[self.resource]['policy_actions'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_action',
                    elem) for elem in parsed_args.actions]

        if parsed_args.classifier:
            body[self.resource]['policy_classifier_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_classifier',
                    parsed_args.classifier)

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description',
                                'enabled', 'shared'])
        return body


class ListPolicyRuleSet(neutronV20.ListCommand):
    """List policy_rule_sets that belong to a given tenant."""

    resource = 'policy_rule_set'
    log = logging.getLogger(__name__ + '.ListPolicyRuleSet')
    _formatters = {}
    list_columns = ['id', 'name', 'policy_rules']
    pagination_support = True
    sorting_support = True


class ShowPolicyRuleSet(neutronV20.ShowCommand):
    """Show information of a given policy_rule_set."""

    resource = 'policy_rule_set'
    log = logging.getLogger(__name__ + '.ShowPolicyRuleSet')


class CreatePolicyRuleSet(neutronV20.CreateCommand):
    """Create a policy rule set for a given tenant."""

    resource = 'policy_rule_set'
    log = logging.getLogger(__name__ + '.CreatePolicyRuleSet')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the policy rule set'))
        parser.add_argument(
            '--policy-rules', type=utils.str2list,
            help=_('Comma separated list of Policy Rules'))
        parser.add_argument(
            '--child-policy-rule-sets', type=utils.str2list,
            help=_('Comma separated list of child Policy Rule Sets'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of Policy Rule Set to create (required argument)'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.policy_rules:
            body[self.resource]['policy_rules'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_rule',
                    elem) for elem in parsed_args.policy_rules]

        if parsed_args.child_policy_rule_sets:
            body[self.resource]['child_policy_rule_sets'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_rule_set',
                    elem) for elem in parsed_args.child_policy_rule_sets]

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description', 'shared'])
        return body


class DeletePolicyRuleSet(neutronV20.DeleteCommand):
    """Delete a given policy rule set."""

    resource = 'policy_rule_set'
    log = logging.getLogger(__name__ + '.DeletePolicyRuleSet')


class UpdatePolicyRuleSet(neutronV20.UpdateCommand):
    """Update policy rule set's information."""

    resource = 'policy_rule_set'
    log = logging.getLogger(__name__ + '.UpdatePolicyRuleSet')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('New name of the Policy Rule Set'))
        parser.add_argument(
            '--description',
            help=_('New description of the Policy Rule Set'))
        parser.add_argument(
            '--policy-rules', type=utils.str2list,
            help=_('New comma separated list of Policy Rules '
                   '(to unset use "")'))
        parser.add_argument(
            '--child-policy-rule-sets', type=utils.str2list,
            help=_('New comma separated list of child Policy Rule Sets '
                   '(to unset use "")'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        if parsed_args.policy_rules == []:
            body[self.resource]['policy_rules'] = []
        elif parsed_args.policy_rules:
            body[self.resource]['policy_rules'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_rule',
                    elem) for elem in parsed_args.policy_rules]

        if parsed_args.child_policy_rule_sets == []:
            body[self.resource]['child_policy_rule_sets'] = []
        elif parsed_args.child_policy_rule_sets:
            body[self.resource]['child_policy_rule_sets'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'policy_rule_set',
                    elem) for elem in parsed_args.child_policy_rule_sets]
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description', 'shared'])
        return body


class ListExternalPolicy(neutronV20.ListCommand):
    """List External Policies that belong to a given tenant."""

    resource = 'external_policy'
    log = logging.getLogger(__name__ + '.ListExternalPolicy')
    list_columns = ['id', 'name', 'description', 'shared', 'external_segments']
    pagination_support = True
    sorting_support = True


class ShowExternalPolicy(neutronV20.ShowCommand):
    """Show information of a given External Policy."""

    resource = 'external_policy'
    log = logging.getLogger(__name__ + '.ShowExternalPolicy')


class CreateExternalPolicy(neutronV20.CreateCommand):
    """Create a External Policy for a given tenant."""

    resource = 'external_policy'
    log = logging.getLogger(__name__ + '.CreateExternalPolicy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the External Policy'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of External Policy to create (required argument)'))
        parser.add_argument(
            '--external-segments', type=utils.str2list,
            help=_('Comma separated list of External Segments'))
        parser.add_argument(
            '--provided-policy-rule-sets', type=utils.str2dict,
            help=_('Comma separated list of Policy Rule Sets'))
        parser.add_argument(
            '--consumed-policy-rule-sets', type=utils.str2dict,
            help=_('Comma separated list of Policy Rule Sets'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.provided_policy_rule_sets:
            for key in parsed_args.provided_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.provided_policy_rule_sets[id_key] = (
                    parsed_args.provided_policy_rule_sets.pop(key))

        if parsed_args.consumed_policy_rule_sets:
            for key in parsed_args.consumed_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.consumed_policy_rule_sets[id_key] = (
                    parsed_args.consumed_policy_rule_sets.pop(key))

        if parsed_args.external_segments:
            body[self.resource]['external_segments'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'external_segment',
                    elem) for elem in parsed_args.external_segments]

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'provided_policy_rule_sets',
                                'consumed_policy_rule_sets', 'shared'])

        return body


class DeleteExternalPolicy(neutronV20.DeleteCommand):
    """Delete a given External Policy."""

    resource = 'external_policy'
    log = logging.getLogger(__name__ + '.DeleteExternalPolicy')


class UpdateExternalPolicy(neutronV20.UpdateCommand):
    """Update External Policy's information."""

    resource = 'external_policy'
    log = logging.getLogger(__name__ + '.UpdateExternalPolicy')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the External Policy'))
        parser.add_argument(
            '--name',
            help=_('New name of the External Policy'))
        parser.add_argument(
            '--external-segments', type=utils.str2list,
            help=_('New comma separated list of External Segments '
                   '(to unset use "")'))
        parser.add_argument(
            '--provided-policy-rule-sets', type=utils.str2dict,
            help=_('New comma separated list of Policy Rule Sets '
                   '(to unset use "")'))
        parser.add_argument(
            '--consumed-policy-rule-sets', type=utils.str2dict,
            help=_('New comma separated list of Policy Rule Sets '
                   '(to unset use "")'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.provided_policy_rule_sets:
            for key in parsed_args.provided_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.provided_policy_rule_sets[id_key] = (
                    parsed_args.provided_policy_rule_sets.pop(key))

        if parsed_args.consumed_policy_rule_sets:
            for key in parsed_args.consumed_policy_rule_sets.keys():
                id_key = neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_rule_set',
                    key)
                parsed_args.consumed_policy_rule_sets[id_key] = (
                    parsed_args.consumed_policy_rule_sets.pop(key))

        if parsed_args.external_segments == []:
            body[self.resource]['external_segments'] = []
        elif parsed_args.external_segments:
            body[self.resource]['external_segments'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'external_segment',
                    elem) for elem in parsed_args.external_segments]

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'provided_policy_rule_sets',
                                'consumed_policy_rule_sets', 'shared'])

        return body


class ListExternalSegment(neutronV20.ListCommand):
    """List External Segments that belong to a given tenant."""

    resource = 'external_segment'
    log = logging.getLogger(__name__ + '.ListExternalSegment')
    _formatters = {'external_routes': _format_host_routes, }
    list_columns = ['id', 'name', 'description', 'cidr',
                    'external_routes', 'port_address_translation', 'shared']
    pagination_support = True
    sorting_support = True


class ShowExternalSegment(neutronV20.ShowCommand):
    """Show information of a given External Segment."""

    resource = 'external_segment'
    log = logging.getLogger(__name__ + '.ShowExternalSegment')


class CreateExternalSegment(neutronV20.CreateCommand):
    """Create a External Segment for a given tenant."""

    resource = 'external_segment'
    log = logging.getLogger(__name__ + '.CreateExternalSegment')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the External Segment'))
        parser.add_argument(
            'name',
            help=_('Name of External Segment to create (required argument)'))
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6],
            help=_('IP version, default is 4'))
        parser.add_argument(
            '--cidr',
            help=_('CIDR of External Segment, default is 172.16.0.0/12'))
        parser.add_argument(
            '--external-route', metavar='destination=CIDR,nexthop=IP_ADDR',
            action='append', dest='external_routes', type=utils.str2dict,
            help=_('If no nexthop, use format: destination=CIDR,nexthop '
                   '(this option can be repeated)'))
        n_utils.add_boolean_argument(
            parser, '--port-address-translation',
            dest='port_address_translation',
            help=_('Enable port-based address translation, default is False'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.external_routes:
            eroutes = []
            for er in parsed_args.external_routes:
                if 'nexthop' in er and er['nexthop'] == '':
                    er['nexthop'] = None
                if er:
                    eroutes.append(er)
            body['external_segment']['external_routes'] = eroutes

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'cidr',
                                'port_address_translation', 'shared'])

        return body


class DeleteExternalSegment(neutronV20.DeleteCommand):
    """Delete a given External Segment."""

    resource = 'external_segment'
    log = logging.getLogger(__name__ + '.DeleteExternalSegment')


class UpdateExternalSegment(neutronV20.UpdateCommand):
    """Update External Segment's information."""

    resource = 'external_segment'
    log = logging.getLogger(__name__ + '.UpdateExternalSegment')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the External Segment'))
        parser.add_argument(
            '--name',
            help=_('New name of External Segment'))
        parser.add_argument(
            '--external-route', metavar='destination=CIDR,nexthop=IP_ADDR',
            action='append', dest='external_routes', type=utils.str2dict,
            help=_('If no nexthop, use format: destination=CIDR,nexthop '
                   '(this option can be repeated)'))
        n_utils.add_boolean_argument(
            parser, '--port-address-translation',
            dest='port_address_translation',
            help=_('Enable or disable port-based address translation'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.external_routes == [{}]:
            body[self.resource]['external_routes'] = []
        elif parsed_args.external_routes:
            eroutes = []
            for er in parsed_args.external_routes:
                if 'nexthop' in er and er['nexthop'] == '':
                    er['nexthop'] = None
                if er:
                    eroutes.append(er)
            body[self.resource]['external_routes'] = eroutes

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'cidr',
                                'port_address_translation', 'shared'])

        return body


class ListNatPool(neutronV20.ListCommand):
    """List NAT Pools that belong to a given tenant."""

    resource = 'nat_pool'
    log = logging.getLogger(__name__ + '.ListNatPool')
    list_columns = ['id', 'name', 'description', 'ip_pool',
                    'external_segment_id', 'shared']
    pagination_support = True
    sorting_support = True


class ShowNatPool(neutronV20.ShowCommand):
    """Show information of a given NAT Pool."""

    resource = 'nat_pool'
    log = logging.getLogger(__name__ + '.ShowNatPool')


class CreateNatPool(neutronV20.CreateCommand):
    """Create a NAT Pool for a given tenant."""

    resource = 'nat_pool'
    log = logging.getLogger(__name__ + '.CreateNatPool')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of the NAT Pool'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of NAT Pool to create (required argument)'))
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6],
            help=_('IP version, default is 4'))
        parser.add_argument(
            '--ip-pool',
            help=_('CIDR for NAT Pool'))
        parser.add_argument(
            '--external-segment',
            help=_('External Segment'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing, default is False'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'ip_pool', 'shared'])

        if parsed_args.external_segment:
            body[self.resource]['external_segment_id'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'external_segment',
                    parsed_args.external_segment))

        return body


class DeleteNatPool(neutronV20.DeleteCommand):
    """Delete a given NAT Pool."""

    resource = 'nat_pool'
    log = logging.getLogger(__name__ + '.DeleteNatPool')


class UpdateNatPool(neutronV20.UpdateCommand):
    """Update NAT Pool's information."""

    resource = 'nat_pool'
    log = logging.getLogger(__name__ + '.UpdateNatPool')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('New description of the NAT Pool'))
        parser.add_argument(
            '--name',
            help=_('New name of NAT Pool'))
        parser.add_argument(
            '--external-segment',
            help=_('New External Segment'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Enable or disable resource sharing'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'ip_version', 'ip_pool', 'shared'])

        if parsed_args.external_segment:
            body[self.resource]['external_segment_id'] = (
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'external_segment',
                    parsed_args.external_segment))

        return body



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import json
import logging
import os

from heatclient.common import template_utils

from neutronclient._i18n import _
from neutronclient.common import exceptions as exc
from neutronclient.common import utils as n_utils
from neutronclient.neutron import v2_0 as neutronV20

from gbpclient.common import utils


class ListServiceChainInstance(neutronV20.ListCommand):
    """List service chain instances that belong to a given tenant."""

    resource = 'servicechain_instance'
    log = logging.getLogger(__name__ + '.ListServiceChainInstance')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'servicechain_spec', 'port']
    pagination_support = True
    sorting_support = True


class ShowServiceChainInstance(neutronV20.ShowCommand):
    """Show information of a given service chain instance."""

    resource = 'servicechain_instance'
    log = logging.getLogger(__name__ + '.ShowServiceChainInstance')


class CreateServiceChainInstance(neutronV20.CreateCommand):
    """Create a service chain instance."""

    resource = 'servicechain_instance'
    log = logging.getLogger(__name__ + '.CreateServiceChainInstance')

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            help=_('Name for the Service Chain Instance.'))
        parser.add_argument(
            '--description',
            help=_('Description of the Service Chain Instance.'))
        parser.add_argument(
            '--service-chain-spec', dest='servicechain_spec',
            help=_('Service Chain Spec ID or the Service Chain Spec name'))
        parser.add_argument(
            '--provider-ptg', dest='provider_ptg',
            help=_('Destination Policy Target Group ID of the Provider.'))
        parser.add_argument(
            '--consumer-ptg', dest='consumer_ptg',
            help=_('Source Policy Target Group ID of the Consumer.'))
        parser.add_argument(
            '--param-values', dest='param_values',
            help=_('Name,Value pairs of Service Configuration Parameters for '
                   'Service Chain Node.'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        if parsed_args.servicechain_spec:
            body[self.resource]['servicechain_spec'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'servicechain_spec',
                    parsed_args.servicechain_spec)
        if parsed_args.provider_ptg:
            body[self.resource]['provider_ptg'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_target_group',
                    parsed_args.provider_ptg)
        if parsed_args.consumer_ptg:
            body[self.resource]['consumer_ptg'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_target_group',
                    parsed_args.consumer_ptg)
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description',
                                'param_values'])
        return body


class UpdateServiceChainInstance(neutronV20.UpdateCommand):
    """Update a given service chain instance."""

    resource = 'servicechain_instance'
    log = logging.getLogger(__name__ + '.UpdateServiceChainInstance')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--service-chain-spec', dest='servicechain_spec',
            help=_('Service Chain Spec ID or the Service Chain Spec name'))
        parser.add_argument(
            '--provider-ptg', dest='provider_ptg',
            help=_('Destination Policy Target Group ID of the Provider.'))
        parser.add_argument(
            '--consumer-ptg', dest='consumer_ptg',
            help=_('Source Policy Target Group ID of the Consumer.'))
        parser.add_argument(
            '--param-values', dest='param_values',
            help=_('Name,Value pairs of Service Configuration Parameters for '
                   'Service Chain Node.'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        if parsed_args.servicechain_spec:
            body[self.resource]['servicechain_spec'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'servicechain_spec',
                    parsed_args.servicechain_spec)
        if parsed_args.provider_ptg:
            body[self.resource]['provider_ptg'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_target_group',
                    parsed_args.provider_ptg)
        if parsed_args.consumer_ptg:
            body[self.resource]['consumer_ptg'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'policy_target_group',
                    parsed_args.consumer_ptg)
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description', 'param_values'])
        return body


class DeleteServiceChainInstance(neutronV20.DeleteCommand):
    """Delete a given service chain instance."""

    resource = 'servicechain_instance'
    log = logging.getLogger(__name__ + '.DeleteServiceChainInstance')


class ListServiceProfile(neutronV20.ListCommand):
    """List service profiles that belong to a given tenant."""

    resource = 'service_profile'
    log = logging.getLogger(__name__ + '.ListServiceProfile')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'service_type']
    pagination_support = True
    sorting_support = True


class ShowServiceProfile(neutronV20.ShowCommand):
    """Show information of a given service profile."""

    resource = 'service_profile'
    log = logging.getLogger(__name__ + '.ShowServiceProfile')


class CreateServiceProfile(neutronV20.CreateCommand):
    """Create a service profile."""

    resource = 'service_profile'
    log = logging.getLogger(__name__ + '.CreateServiceProfile')

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            help=_('Name for the Service Profile.'))
        parser.add_argument(
            '--description',
            help=_('Description of the Service Profile.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))
        parser.add_argument(
            '--vendor',
            help=_('Vendor providing the service node'))
        parser.add_argument(
            '--insertion-mode',
            help=_('Insertion mode of the service'))
        parser.add_argument(
            '--servicetype', dest='service_type',
            help=_('Type of the service'))
        parser.add_argument(
            '--service-flavor',
            help=_('Flavor of the service'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description', 'tenant_id', 'shared',
                                'vendor', 'insertion_mode', 'service_type',
                                'service_flavor'])
        return body


class UpdateServiceProfile(neutronV20.UpdateCommand):
    """Update a given service profile."""

    resource = 'service_profile'
    log = logging.getLogger(__name__ + '.UpdateServiceProfile')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Name for the Service Profile.'))
        parser.add_argument(
            '--description',
            help=_('Description of the Service Profile.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))
        parser.add_argument(
            '--vendor',
            help=_('Vendor providing the service node'))
        parser.add_argument(
            '--insertion-mode',
            help=_('Insertion mode of the service'))
        parser.add_argument(
            '--servicetype', dest='service_type',
            help=_('Type of the service'))
        parser.add_argument(
            '--service-flavor',
            help=_('Flavor of the service'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description', 'shared', 'vendor',
                                'insertion_mode', 'service_type',
                                'service_flavor'])
        return body


class DeleteServiceProfile(neutronV20.DeleteCommand):
    """Delete a given service profile."""

    resource = 'service_profile'
    log = logging.getLogger(__name__ + '.DeleteServiceProfile')


class ListServiceChainNode(neutronV20.ListCommand):
    """List service chain nodes that belong to a given tenant."""

    resource = 'servicechain_node'
    log = logging.getLogger(__name__ + '.ListServiceChainNode')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'service_type']
    pagination_support = True
    sorting_support = True


class ShowServiceChainNode(neutronV20.ShowCommand):
    """Show information of a given service chain node."""

    resource = 'servicechain_node'
    log = logging.getLogger(__name__ + '.ShowServiceChainNode')


class CreateServiceChainNode(neutronV20.CreateCommand):
    """Create a service chain node."""

    resource = 'servicechain_node'
    log = logging.getLogger(__name__ + '.CreateServiceChainNode')

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            help=_('Name for the Service Chain Node.'))
        parser.add_argument(
            '--description',
            help=_('Description of the Service Chain Node.'))
        parser.add_argument(
            '--servicetype', dest='service_type',
            help=_('Service type ID or the Service Type name'))
        parser.add_argument(
            '--service-profile',
            help=_('Service Profile name or UUID'))
        parser.add_argument(
            '--config',
            help=_('Service Configuration for the Service Chain Node.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))
        parser.add_argument(
            '--template-file',
            help=_('Service Configuration Template for the Service Chain '
                   'Node.'))
        parser.add_argument(
            '--param-names', dest='param_names',
            help=_('List of Configuration Parameter Names for Service '
                   'Chain Node.'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        if parsed_args.service_profile:
            body[self.resource]['service_profile_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'service_profile',
                    parsed_args.service_profile)
        if parsed_args.template_file:
            if os.path.isfile(parsed_args.template_file):
                tpl_files, template = template_utils.get_template_contents(
                    parsed_args.template_file)
                parsed_args.config = json.dumps(template)
            else:
                raise exc.NeutronClientException("File %s does not exist. "
                                                 "Please check the path"
                                                 % parsed_args.template_file)
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'service_type', 'config', 'shared',
                                'tenant_id', 'param_names', 'description'])
        return body


class UpdateServiceChainNode(neutronV20.UpdateCommand):
    """Update a given service chain node."""

    resource = 'servicechain_node'
    log = logging.getLogger(__name__ + '.UpdateServiceChainNode')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--servicetype', dest='service_type',
            help=_('Service type ID or the Service Type name'))
        parser.add_argument(
            '--service-profile',
            help=_('Service Profile name or UUID'))
        parser.add_argument(
            '--config',
            help=_('Service Configuration for the Service Chain Node.'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }
        if parsed_args.service_profile:
            body[self.resource]['service_profile_id'] = \
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(), 'service_profile',
                    parsed_args.service_profile)
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'service_type', 'config', 'shared',
                                'description'])
        return body


class DeleteServiceChainNode(neutronV20.DeleteCommand):
    """Delete a given service chain node."""

    resource = 'servicechain_node'
    log = logging.getLogger(__name__ + '.DeleteServiceChainNode')


class ListServiceChainSpec(neutronV20.ListCommand):
    """List service chain specs that belong to a given tenant."""

    resource = 'servicechain_spec'
    log = logging.getLogger(__name__ + '.ListServiceChainSpec')
    _formatters = {}
    list_columns = ['id', 'name', 'description', 'nodes']
    pagination_support = True
    sorting_support = True


class ShowServiceChainSpec(neutronV20.ShowCommand):
    """Show information of a given service chain spec."""

    resource = 'servicechain_spec'
    log = logging.getLogger(__name__ + '.ShowServiceChainSpec')


class CreateServiceChainSpec(neutronV20.CreateCommand):
    """Create a service chain spec."""

    resource = 'servicechain_spec'
    log = logging.getLogger(__name__ + '.CreateServiceChainSpec')

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            help=_('Name for the Service Chain Spec.'))
        parser.add_argument(
            '--description',
            help=_('Description of the Service Chain Specification.'))
        parser.add_argument(
            '--nodes', metavar='NODES', type=utils.str2list,
            help=_('Comma separated list of Service Chain Nodes'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.nodes:
            body[self.resource]['nodes'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'servicechain_node',
                    elem) for elem in parsed_args.nodes]

        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'tenant_id', 'description', 'shared'])
        return body


class UpdateServiceChainSpec(neutronV20.UpdateCommand):
    """Update a given service chain spec."""

    resource = 'servicechain_spec'
    log = logging.getLogger(__name__ + '.UpdateServiceChainSpec')

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--nodes', type=utils.str2list,
            help=_('New comma separated list of Service Chain Nodes '
                   '(to unset use "")'))
        n_utils.add_boolean_argument(
            parser, '--shared', dest='shared',
            help=_('Shared flag'))

    def args2body(self, parsed_args):
        body = {self.resource: {}, }

        if parsed_args.nodes == []:
            body[self.resource]['nodes'] = []
        elif parsed_args.nodes:
            body[self.resource]['nodes'] = [
                neutronV20.find_resourceid_by_name_or_id(
                    self.get_client(),
                    'servicechain_node',
                    elem) for elem in parsed_args.nodes]
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['name', 'description', 'shared'])
        return body


class DeleteServiceChainSpec(neutronV20.DeleteCommand):
    """Delete a given service chain spec."""

    resource = 'servicechain_spec'
    log = logging.getLogger(__name__ + '.DeleteServiceChainSpec')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging
import sys

from gbpclient.gbp.v2_0 import groupbasedpolicy as gbp
from gbpclient.tests.unit import test_cli20


class CLITestV20L3PolicyJSON(test_cli20.CLITestV20Base):

    LOG = logging.getLogger(__name__)

    def setUp(self):
        super(CLITestV20L3PolicyJSON, self).setUp()

    def test_create_l3_policy_with_mandatory_params(self):
        resource = 'l3_policy'
        cmd = gbp.CreateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--tenant-id', tenant_id,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id)

    def test_create_l3_policy_with_all_params(self):
        """l3-policy-create with all params."""
        resource = 'l3_policy'
        cmd = gbp.CreateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        tenant_id = 'mytenant'
        description = 'My L3 Policy'
        my_id = 'someid'
        ip_version = '4'
        ip_pool = '172.16.0.0/12'
        subnet_prefix_length = '24'
        address_scope_v4_id = 'ascpid'
        subnetpools_v4 = 'sp1,sp2'
        external_segment = 'seg_uuid1=1.1.1.0:2.2.2.0'
        expected_external_segments = {'seg_uuid1': ['1.1.1.0', '2.2.2.0']}
        routers = 'uuid1,uuid2'
        shared = 'true'
        args = ['--tenant-id', tenant_id,
                '--description', description,
                '--ip-version', ip_version,
                '--ip-pool', ip_pool,
                '--subnet-prefix-length', subnet_prefix_length,
                '--address-scope-v4-id', address_scope_v4_id,
                '--subnetpools-v4', subnetpools_v4,
                '--external-segment', external_segment,
                '--routers', routers,
                '--shared', shared,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id,
                                   description=description,
                                   ip_version=4,
                                   ip_pool=ip_pool,
                                   subnet_prefix_length=24,
                                   address_scope_v4_id=address_scope_v4_id,
                                   subnetpools_v4=['sp1', 'sp2'],
                                   routers=['uuid1', 'uuid2'],
                                   external_segments=
                                   expected_external_segments, shared=shared)

    def test_create_l3_policy_with_ipv6(self):
        """l3-policy-create with ipv6 params."""
        resource = 'l3_policy'
        cmd = gbp.CreateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        tenant_id = 'mytenant'
        description = 'My L3 Policy'
        my_id = 'someid'
        ip_version = '6'
        address_scope_v6_id = 'ascpid'
        subnetpools_v6 = 'sp1,sp2'
        args = ['--tenant-id', tenant_id,
                '--description', description,
                '--ip-version', ip_version,
                '--address-scope-v6-id', address_scope_v6_id,
                '--subnetpools-v6', subnetpools_v6,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id,
                                   description=description,
                                   ip_version=6,
                                   address_scope_v6_id=address_scope_v6_id,
                                   subnetpools_v6=['sp1', 'sp2'])

    def test_create_l3_policy_with_external_segment(self):
        """l3-policy-create with all params."""
        resource = 'l3_policy'
        cmd = gbp.CreateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'name'
        tenant_id = 'mytenant'
        my_id = 'someid'
        external_segment = 'seg_uuid1'
        expected_external_segments = {'seg_uuid1': []}
        args = ['--tenant-id', tenant_id,
                '--external-segment', external_segment,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id,
                                   external_segments=
                                   expected_external_segments)

    def test_create_l3_policy_with_allowed_vm_names(self):
        resource = 'l3_policy'
        cmd = gbp.CreateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'name'
        tenant_id = 'mytenant'
        my_id = 'someid'
        allowed_vm_names = "^safe_vm*,good_vm*"
        args = ['--tenant-id', tenant_id,
                '--allowed-vm-names', allowed_vm_names,
                name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   tenant_id=tenant_id,
                                   allowed_vm_names=['^safe_vm*', 'good_vm*'])

    def test_list_l3_policies(self):
        resource = 'l3_policies'
        cmd = gbp.ListL3Policy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resource, cmd, True)

    def test_show_l3_policy(self):
        resource = 'l3_policy'
        cmd = gbp.ShowL3Policy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_l3_policy(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_l3_policy_with_all_params(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        description = 'My L3 Policy'
        my_id = 'someid'
        subnet_prefix_length = '24'
        external_segment = 'seg_uuid1=1.1.1.0:2.2.2.0'
        expected_external_segments = {'seg_uuid1': ['1.1.1.0', '2.2.2.0']}
        shared = 'true'
        subnetpools_v4 = 'sp1,sp2'
        routers = 'uuid1,uuid2'
        args = ['--name', name,
                '--description', description,
                '--subnet-prefix-length', subnet_prefix_length,
                '--subnetpools-v4', subnetpools_v4,
                '--external-segment', external_segment,
                '--routers', routers,
                '--shared', shared,
                my_id]
        params = {
            'name': name,
            'description': description,
            'subnet_prefix_length': 24,
            'subnetpools_v4': ['sp1', 'sp2'],
            'external_segments': expected_external_segments,
            'routers': routers,
            'routers': ['uuid1', 'uuid2'],
            'shared': shared
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_l3_policy_ipv6_subnetpools(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'someid'
        subnetpools_v6 = 'sp1,sp2'
        args = ['--subnetpools-v6', subnetpools_v6,
                my_id]
        params = {
            'subnetpools_v6': ['sp1', 'sp2'],
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_l3_policy_unset_external_segment(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        description = 'My L3 Policy'
        my_id = 'someid'
        subnet_prefix_length = '24'
        external_segment = ''
        expected_external_segments = {}
        args = ['--name', name,
                '--description', description,
                '--subnet-prefix-length', subnet_prefix_length,
                '--external-segment', external_segment,
                my_id]
        params = {
            'name': name,
            'description': description,
            'subnet_prefix_length': 24,
            'external_segments': expected_external_segments,
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_l3_policy_unset_routers(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'someid'
        routers = ''
        args = ['--routers', routers,
                my_id]
        params = {
            'routers': [],
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_update_l3_policy_with_allowed_vm_names(self):
        resource = 'l3_policy'
        cmd = gbp.UpdateL3Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'someid'
        allowed_vm_names = "bad_vm*,^worse_vm*"
        args = ['--allowed-vm-names', allowed_vm_names,
                my_id]
        params = {
            'allowed_vm_names': ['bad_vm*', '^worse_vm*'],
        }
        self._test_update_resource(resource, cmd, my_id, args, params)

    def test_delete_l3_policy_name(self):
        resource = 'l3_policy'
        cmd = gbp.DeleteL3Policy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)
